unit DayBlockView;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, DsgnIntf, DayList, Grids, Menus;
{**************************************************************************************************}
Type
TFrmDayBlockDlg = class(TForm)
   OKBtn: TButton;
   CancelBtn: TButton;
   BtnAdd: TButton;
   BtnEdit: TButton;
   BtnDelete: TButton;
   DayGrid: TStringGrid;
    PopupMenu1: TPopupMenu;
    Add1: TMenuItem;
    Edit1: TMenuItem;
    Delete1: TMenuItem;
    procedure Add1Click(Sender: TObject);
    procedure Edit1Click(Sender: TObject);
    procedure Delete1Click(Sender: TObject);
    procedure BtnAddClick(Sender: TObject);
    procedure BtnEditClick(Sender: TObject);
    procedure BtnDeleteClick(Sender: TObject);
    procedure DayGridDblClick(Sender: TObject);
private
   m_TDayList: TDayList;

   function  Init(aList: TDayList): Boolean;
   procedure Deinit;

   procedure AddDayListToGrid;
   procedure DeleteDayListFromGrid;
   procedure UpdateDayList;

   procedure DoNew;
   procedure DoEdit;
   procedure DoDelete;

  public
     function Execute(aList: TDayList): Boolean;
  end;
{**************************************************************************************************}
var
  FrmDayBlockDlg: TFrmDayBlockDlg;
{**************************************************************************************************}
implementation
Uses Dialogs ,
     Plan    ,
     Utils   ,
     FrmDayIODlg;

{$R *.DFM}
{**************************************************************************************************}
function  TFrmDayBlockDlg.Execute(aList: TDayList): Boolean;
begin
   Result := FALSE;
   If(Init(aList)) then
      Result := (ShowModal = mrOk);
   Deinit;
end;
{**************************************************************************************************}
function  TFrmDayBlockDlg.Init(aList: TDayList): Boolean;
begin
   try
      Result := Assigned(aList);
      m_TDayList := aList;
      AddDayListToGrid;
   except
      on E:Exception do begin
         Result := FALSE;
         MessageDlg('Error while initilizing dialog.'+#13+#10+E.Message, mtError, [mbOk] ,0);
      end;
   end;
end;
{**************************************************************************************************}
procedure TFrmDayBlockDlg.Deinit;
begin
   m_TDayList := NIL;  
end;
{**************************************************************************************************}
procedure TFrmDayBlockDlg.AddDayListToGrid;
var
  nCounter: integer;
begin
   DayGrid.Cells[0,0] := 'Date:';
   DayGrid.Cells[1,0] := 'Day string:';
   DayGrid.RowCount := lnMax(m_TDayList.Count+1, 2);

   for nCounter := 0 to m_TDayList.Count-1 do begin
      DayGrid.Cells[0,nCounter+1] := DateToStr(m_TDayList.Items[nCounter].Day);
      DayGrid.Cells[1,nCounter+1] := m_TDayList.Items[nCounter].DisplayString;
   end;
   
end;
{**************************************************************************************************}
procedure TFrmDayBlockDlg.DeleteDayListFromGrid;
var
  nCounter: integer;
begin
   for nCounter := 1 to DayGrid.RowCount - 1 do 
      DayGrid.Rows[nCounter].Clear;
end;
{**************************************************************************************************}
procedure TFrmDayBlockDlg.UpdateDayList;
begin
   DeleteDayListFromGrid;
   AddDayListToGrid;
end;
{**************************************************************************************************}
procedure TFrmDayBlockDlg.DoNew;
var
  aDay : TDay;
begin
   try
      aDay := TDay.CreateWithParam(now,'New', NIL);
      FrmDayIODialog := TFrmDayIODialog.Create(Self);
      try
         if(FrmDayIODialog.Execute(aDay)) then begin
            if(m_TDayList.CheckForOverlapping) then begin
               if(m_TDayList.IsDateInList(aDay.day)<>-1) then
                  MessageDlg('Date is allready in list.', mtInformation, [mbOk] ,0)
               else
                  m_TDayList.Add(aDay);
            end
            else
               m_TDayList.Add(aDay);
         end;
      finally
         FrmDayIODialog.Free;
      end;
   except 
      on E:Exception do
         MessageDlg('Error building dialog.'+#13+#10+E.Message, mtError, [mbOk] ,0);
   end;
   UpdateDayList;
end;
{**************************************************************************************************}
procedure TFrmDayBlockDlg.DoEdit;
var
  aDay    : TDay;
  aTmpDay : TDay;
begin
   if(m_TDayList.Count = 0) then begin
      MessageDlg('No days to edit.', mtInformation, [mbOk] ,0);
      DoNew;
      Exit;
   end;

   aDay := m_TDayList.Items[DayGrid.Row-1];
   aTmpDay:= TDay.CreateWithObj(aDay);
   try
      FrmDayIODialog := TFrmDayIODialog.Create(Self);
      try
         if(FrmDayIODialog.Execute(aTmpDay)) then begin
            if(aTmpDay.Day <> aDay.Day) then begin
               if(m_TDayList.CheckForOverlapping) then begin
                  if(m_TDayList.IsDateInList(aTmpDay.Day)<>-1) then
                     MessageDlg('Date is allready in list.', mtInformation, [mbOk] ,0)
                  else begin
                     m_TDayList.Delete(DayGrid.Row-1);
                     m_TDayList.Add(TDay.CreateWithObj(aTmpDay));
                  end;
               end
               else begin
                  m_TDayList.Delete(DayGrid.Row-1);
                  m_TDayList.Add(TDay.CreateWithObj(aTmpDay));
               end;
            end
            else
               aDay.DisplayString := aTmpDay.DisplayString;
         end;
      finally
         FrmDayIODialog.Free
      end;
   finally
      aTmpDay.Free;
   end;
   UpdateDayList;
end;
{**************************************************************************************************}
procedure TFrmDayBlockDlg.DoDelete;
begin
   if(m_TDayList.Count = 0) then begin
      MessageDlg('No days to delete.', mtInformation, [mbOk] ,0);
      exit;
   end;

   m_TDayList.Delete(DayGrid.Row-1);
   UpdateDayList;
end;
{**************************************************************************************************}
{**************************************************************************************************}
{* Eigene Funktionen (ENDE)                                                                       *}
{**************************************************************************************************}
procedure TFrmDayBlockDlg.Add1Click(Sender: TObject);
begin
   DoNew;
end;
{**************************************************************************************************}
procedure TFrmDayBlockDlg.Edit1Click(Sender: TObject);
begin
   DoEdit;
end;
{**************************************************************************************************}
procedure TFrmDayBlockDlg.Delete1Click(Sender: TObject);
begin
   DoDelete;
end;
{**************************************************************************************************}
procedure TFrmDayBlockDlg.BtnAddClick(Sender: TObject);
begin
   DoNew;
end;
{**************************************************************************************************}
procedure TFrmDayBlockDlg.BtnEditClick(Sender: TObject);
begin
   DoEdit;
end;
{**************************************************************************************************}
procedure TFrmDayBlockDlg.BtnDeleteClick(Sender: TObject);
begin
   DoDelete;
end;
{**************************************************************************************************}
procedure TFrmDayBlockDlg.DayGridDblClick(Sender: TObject);
begin
   DoEdit;
end;
{**************************************************************************************************}
end.
